@@ -21,17 +21,18 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 import datetime
 import weakref
 import os
-import csv
 if os.name != 'nt':
     import resource
-from random import Random
 from collections import defaultdict
 from functools import total_ordering
 
 from twisted.internet.defer import succeed
-from util import stack, gitHead
+from util import gitHead
+from rand import CountingRandom
 from log import logError, logWarning, logException, logDebug, m18n
-from common import WINDS, Internal, IntDict, Debug, Options, unicodeString
+from common import Internal, IntDict, Debug, Options, unicodeString, unicode
+from common import isPython3
+from wind import Wind, East
 from query import Query
 from rule import Ruleset
 from tile import Tile, elements
@@ -39,79 +40,7 @@ from sound import Voice
 from wall import Wall
 from move import Move
 from player import Players, Player, PlayingPlayer
-
-
-class CountingRandom(Random):
-
-    """counts how often random() is called and prints debug info"""
-
-    def __init__(self, game, value=None):
-        self._game = weakref.ref(game)
-        Random.__init__(self, value)
-        self.count = 0
-
-    @property
-    def game(self):
-        """hide the fact that game is a weakref"""
-        return self._game()
-
-    def random(self):
-        """the central randomizator"""
-        self.count += 1
-        return Random.random(self)
-
-    def seed(self, newSeed=None):
-        self.count = 0
-        if Debug.random:
-            self.game.debug('Random gets seed %s' % newSeed)
-        Random.seed(self, newSeed)
-
-    def shuffle(self, listValue, func=None):
-        """pylint needed for python up to 2.7.5"""
-        # pylint: disable=arguments-differ
-        oldCount = self.count
-        Random.shuffle(self, listValue, func)
-        if Debug.random:
-            self.game.debug(
-                '%d out of %d calls to random by Random.shuffle from %s'
-                % (self.count - oldCount, self.count, stack('')[-2]))
-
-    def randrange(self, start, stop=None, step=1):
-        # pylint: disable=arguments-differ
-        oldCount = self.count
-        result = Random.randrange(self, start, stop, step)
-        if Debug.random:
-            self.game.debug(
-                '%d out of %d calls to random by '
-                'Random.randrange(%d,%s) from %s'
-                % (self.count - oldCount, self.count, start, stop,
-                   stack('')[-2]))
-        return result
-
-    def choice(self, fromList):
-        if len(fromList) == 1:
-            return fromList[0]
-        oldCount = self.count
-        result = Random.choice(self, fromList)
-        if Debug.random:
-            self.game.debug(
-                '%d out of %d calls to random by '
-                'Random.choice(%s) from %s' % (
-                    self.count - oldCount, self.count,
-                    str([str(x) for x in fromList]),
-                    stack('')[-2]))
-        return result
-
-    def sample(self, population, wantedLength):
-        oldCount = self.count
-        result = Random.sample(self, population, wantedLength)
-        if Debug.random:
-            self.game.debug(
-                '%d out of %d calls to random by '
-                'Random.sample(x, %d) from %s' %
-                (self.count - oldCount, self.count, wantedLength,
-                 stack('')[-2]))
-        return result
+from compat import CsvWriter
 
 
 @total_ordering
@@ -144,7 +73,7 @@ class HandId(object):
         stringIdx 0 is the part in front of ..
         stringIdx 1 is the part after ..
         """
-        # pylint: disable=too-many-return-statements
+        # pylint: disable=too-many-return-statements,too-many-branches
         if not string:
             return
         seed = int(string.split('/')[0])
@@ -154,8 +83,10 @@ class HandId(object):
             if stringIdx == 1:
                 self.roundsFinished = 100
             return
-        string = string.split('/')[1]
-        parts = string.split('..')
+        string1 = string.split('/')[1]
+        if not string1:
+            logException('--game=%s must specify the wanted round' % string)
+        parts = string1.split('..')
         if stringIdx == 1 and len(parts) == 2 and parts[1] == '':
             self.roundsFinished = 100
             return
@@ -165,16 +96,16 @@ class HandId(object):
             self.roundsFinished = 100
             return
         handId = parts[min(stringIdx, len(parts) - 1)]
-        if handId[0] not in WINDS:
-            logException('--game=%s with / must specify the round wind'
-                         % string)
+        if handId[0].lower() not in 'eswn':
+            logException('--game=%s must specify the round wind' % string)
+        handWind = Wind(handId[0])
         ruleset = self.game.ruleset
-        self.roundsFinished = WINDS.index(handId[0])
+        self.roundsFinished = handWind.__index__()
         if self.roundsFinished > ruleset.minRounds:
             logWarning(
                 u'Ruleset %s has %d minimum rounds but you want round %d(%s)'
                 % (ruleset.name, ruleset.minRounds, self.roundsFinished + 1,
-                   handId[0]))
+                   handWind))
             self.roundsFinished = ruleset.minRounds
             return
         self.rotated = int(handId[1]) - 1
@@ -222,16 +153,18 @@ class HandId(object):
         while num:
             charId = chr(ord('a') + (num - 1) % 26) + charId
             num = (num - 1) // 26
-        wind = (WINDS + 'X')[self.roundsFinished]
+        if not charId:
+            charId = ' ' # align to the most common case
+        wind = Wind.all4[self.roundsFinished % 4]
         if withSeed:
-            seed = self.seed
+            seed = str(self.seed)
         else:
             seed = ''
         delim = '/' if withSeed or withAI else ''
         result = '%s%s%s%s%s%s' % (
             aiVariant, seed, delim, wind, self.rotated + 1, charId)
         if withMoveCount:
-            result += '/%d' % self.moveCount
+            result += '/%3d' % self.moveCount
         return result
 
     def token(self):
@@ -251,7 +184,7 @@ class HandId(object):
                 (other.roundsFinished, other.rotated, other.notRotated))
 
     def __ne__(self, other):
-        return not self == other
+        return not self == other # pylint: disable=unneeded-not
 
     def __lt__(self, other):
         return (self.roundsFinished, self.rotated, self.notRotated) < (
@@ -276,6 +209,9 @@ class Game(object):
         """
         # pylint: disable=too-many-statements
         assert self.__class__ != Game, 'Do not directly instantiate Game'
+        for wind, name in names:
+            assert isinstance(wind, Wind), 'Game.__init__ expects Wind objects'
+            assert isinstance(name, (str, unicode)), 'Game.__init__: name must be string and not {}'.format(type(name))
         self.players = Players()
         # if we fail later on in init, at least we can still close the program
         self.myself = None
@@ -397,7 +333,7 @@ class Game(object):
     @property
     def roundWind(self):
         """the round wind for Hand"""
-        return 'eswn'[self.roundsFinished % 4]
+        return Wind.all[self.roundsFinished % 4]
 
     @winner.setter
     def winner(self, value):
@@ -484,17 +420,16 @@ class Game(object):
         """assign random seats to the players and assign winds"""
         self.players.sort(key=lambda x: x.name)
         self.randomGenerator.shuffle(self.players)
-        for player, wind in zip(self.players, WINDS):
+        for player, wind in zip(self.players, Wind.all4):
             player.wind = wind
 
     def __exchangeSeats(self):
         """execute seat exchanges according to the rules"""
-        winds = self.shiftRules.split(',')[(self.roundsFinished - 1) % 4]
-        players = list(self.players[x] for x in winds)
+        winds = list(x for x in self.shiftRules.split(',')[(self.roundsFinished - 1) % 4])
+        players = list(self.players[Wind(x)] for x in winds)
         pairs = list(players[x:x + 2] for x in range(0, len(winds), 2))
         for playerA, playerB in self._mustExchangeSeats(pairs):
             playerA.wind, playerB.wind = playerB.wind, playerA.wind
-        self.sortPlayers()
 
     def _mustExchangeSeats(self, pairs):
         """filter: which player pairs should really swap places?"""
@@ -503,8 +438,8 @@ class Game(object):
 
     def sortPlayers(self):
         """sort by wind order. Place ourself at bottom (idx=0)"""
-        self.players.sort(key=lambda x: WINDS.index(x.wind))
-        self.activePlayer = self.players[u'E']
+        self.players.sort(key=lambda x: x.wind)
+        self.activePlayer = self.players[East]
         if Internal.scene:
             if self.belongsToHumanPlayer():
                 while self.players[0] != self.myself:
@@ -522,7 +457,7 @@ class Game(object):
         """save starttime for this game"""
         starttime = datetime.datetime.now().replace(microsecond=0).isoformat()
         args = list([starttime, self.seed, int(self.autoPlay),
-                    self.ruleset.rulesetId])
+                     self.ruleset.rulesetId])
         args.extend([p.nameid for p in self.players])
         args.append(self.gameid)
         Query("update game set starttime=?,seed=?,autoplay=?,"
@@ -594,7 +529,7 @@ class Game(object):
         """save computed values to database,
         update score table and balance in status line"""
         scoretime = datetime.datetime.now().replace(microsecond=0).isoformat()
-        logMessage = ''
+        logMessage = u''
         for player in self.players:
             if player.hand:
                 manualrules = '||'.join(x.rule.name
@@ -607,13 +542,13 @@ class Game(object):
                 "wind,points,payments, balance,rotated,notrotated) "
                 "VALUES(%d,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" %
                 (self.gameid, self.handctr, player.nameid,
-                    scoretime, int(player == self.__winner),
-                    WINDS[self.roundsFinished % 4], player.wind,
-                    player.handTotal, player.payment, player.balance,
-                    self.rotated, self.notRotated),
+                 scoretime, int(player == self.__winner),
+                 self.roundWind.char, player.wind,
+                 player.handTotal, player.payment, player.balance,
+                 self.rotated, self.notRotated),
                 (player.hand.string, manualrules))
-            logMessage += '{player:<12} {hand:>4} {total:>5} {won} | '.format(
-                player=str(player)[:12], hand=player.handTotal,
+            logMessage += u'{player:<12} {hand:>4} {total:>5} {won} | '.format(
+                player=unicode(player)[:12], hand=player.handTotal,
                 total=player.balance,
                 won='WON' if player == self.winner else '   ')
             for usedRule in player.hand.usedRules:
@@ -683,7 +618,7 @@ class Game(object):
         handId = self._prevHandId if prevHandId else self.handId
         handId = unicodeString(handId.prompt(withMoveCount=True))
         logDebug(
-            u'%s%s: %s' % (prefix, handId, msg),
+            u'%s%s: %s' % (prefix, handId, unicodeString(msg)),
             withGamePrefix=False,
             btIndent=btIndent)
 
@@ -700,21 +635,21 @@ class Game(object):
     def loadFromDB(cls, gameid, client=None):
         """load game by game id and return a new Game instance"""
         Internal.logPrefix = 'S' if Internal.isServer else 'C'
-        qGameRecords = Query(
+        records = Query(
             "select p0,p1,p2,p3,ruleset,seed from game where id = ?",
             (gameid,)).records
-        if not qGameRecords:
+        if not records:
             return None
-        qGameRecord = qGameRecords[0]
+        qGameRecord = records[0]
         rulesetId = qGameRecord[4] or 1
         ruleset = Ruleset.cached(rulesetId)
         Players.load()  # we want to make sure we have the current definitions
-        qLastHandRecords = Query(
+        records = Query(
             "select hand,rotated from score where game=? and hand="
             "(select max(hand) from score where game=?)",
             (gameid, gameid)).records
-        if qLastHandRecords:
-            qLastHandRecord = qLastHandRecords[0]
+        if records:
+            qLastHandRecord = records[0]
         else:
             qLastHandRecord = tuple([0, 0])
         qScoreRecords = Query(
@@ -723,9 +658,9 @@ class Game(object):
             (gameid, qLastHandRecord[0])).records
         if not qScoreRecords:
             # this should normally not happen
-            qScoreRecords = tuple([
-                tuple([qGameRecord[x], WINDS[x], 0, False, 'E'])
-                for x in range(4)])
+            qScoreRecords = list([
+                tuple([qGameRecord[wind], wind.char, 0, False, East.char])
+                for wind in Wind.all4])
         if len(qScoreRecords) != 4:
             logError(u'game %d inconsistent: There should be exactly '
                      '4 score records for the last hand' % gameid)
@@ -738,9 +673,8 @@ class Game(object):
         if len(set(x[4] for x in qScoreRecords)) != 1:
             logError(u'game %d inconsistent: All score records for the same '
                      'hand must have the same prevailing wind' % gameid)
-        prevailing = qScoreRecords[0][4]
 
-        players = list(tuple([x[1], Game.__getName(x[0])])
+        players = list(tuple([Wind(x[1]), Game.__getName(x[0])])
                        for x in qScoreRecords)
 
         # create the game instance.
@@ -759,7 +693,7 @@ class Game(object):
                 player.getsPayment(record[2])
             if record[3]:
                 game.winner = player
-        game.roundsFinished = WINDS.index(prevailing)
+        game.roundsFinished = Wind(qScoreRecords[0][4]).__index__()
         game.handctr += 1
         game.notRotated += 1
         game.maybeRotateWinds()
@@ -775,7 +709,7 @@ class Game(object):
         if self.handId > last:
             return True
         if Options.rounds:
-            return self.roundsFinished >= 1
+            return self.roundsFinished >= Options.rounds
         elif self.ruleset:
             # while initialising Game, ruleset might be None
             return self.roundsFinished >= self.ruleset.minRounds
@@ -796,7 +730,7 @@ class Game(object):
                                (self.handId, winner, guilty))
                 guilty.hand.usedRules.append((payAction, None))
                 score = winner.handTotal
-                score = score * 6 if winner.wind == 'E' else score * 4
+                score = score * 6 if winner.wind == East else score * 4
                 guilty.getsPayment(-score)
                 winner.getsPayment(score)
                 return
@@ -809,7 +743,7 @@ class Game(object):
                         self.debug('   %s' % (line))
             for player2 in self.players:
                 if id(player1) != id(player2):
-                    if player1.wind == 'E' or player2.wind == 'E':
+                    if player1.wind == East or player2.wind == East:
                         efactor = 2
                     else:
                         efactor = 1
@@ -878,16 +812,17 @@ class PlayingGame(Game):
         """write game summary to Options.csv"""
         if self.finished() and Options.csv:
             gameWinner = max(self.players, key=lambda x: x.balance)
-            writer = csv.writer(open(Options.csv, 'ab'), delimiter=';')
+            writer = CsvWriter(Options.csv, mode='a')
             if Debug.process and os.name != 'nt':
                 self.csvTags.append('MEM:%s' % resource.getrusage(
                     resource.RUSAGE_SELF).ru_maxrss)
             if Options.rounds:
                 self.csvTags.append('ROUNDS:%s' % Options.rounds)
-            row = [self.ruleset.name, Options.AI, gitHead(), str(self.seed),
-                   ','.join(self.csvTags)]
+            row = [self.ruleset.name, Options.AI,
+                   gitHead(), '3' if isPython3 else '2',
+                   str(self.seed), ','.join(self.csvTags)]
             for player in sorted(self.players, key=lambda x: x.name):
-                row.append(player.name.encode('utf-8'))
+                row.append(player.name)
                 row.append(player.balance)
                 row.append(player.wonCount)
                 row.append(1 if player == gameWinner else 0)
@@ -948,7 +883,7 @@ class PlayingGame(Game):
             # in server.__prepareNewGame, gameid is None here
             return
         records = Query("select seed from game where id=?", (
-                        self.gameid,)).records
+            self.gameid,)).records
         assert records, 'self.gameid: %s' % self.gameid
         seed = records[0][0]
 
@@ -1057,13 +992,13 @@ class PlayingGame(Game):
                 # and have a voice
                 if Debug.sound and player != self.myself:
                     logDebug(u'%s got voice from opponent: %s' % (
-                             player.name, player.voice))
+                        player.name, player.voice))
             else:
                 player.voice = Voice.locate(player.name)
                 if player.voice:
                     if Debug.sound:
                         logDebug(u'%s has own local voice %s' % (
-                                 player.name, player.voice))
+                            player.name, player.voice))
             if player.voice:
                 for voice in Voice.availableVoices():
                     if (voice in available
