@@ -19,6 +19,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from common import Internal, ZValues, unicode
+from wind import East
 from qt import QRectF, QPointF, QGraphicsSimpleTextItem, QFontMetrics
 
 from board import PlayerWind, YellowText, Board, rotateCenter
@@ -111,7 +112,7 @@ class UIWall(Wall):
             side.setParentItem(self.__square)
             side.lightSource = self.lightSource
             side.windTile = PlayerWind(
-                'E',
+                East,
                 Internal.scene.windTileset,
                 parent=side)
             side.windTile.hide()
@@ -244,7 +245,7 @@ class UIWall(Wall):
         for side in self.__sides:
             side.nameLabel.setFont(font)
 
-    def showShadowsChanged(self, oldValue, newValue):
+    def showShadowsChanged(self, dummyOldValue, dummyNewValue):
         """setting this actually changes the visuals."""
         assert ParallelAnimationGroup.current is None
         self.__resizeHandBoards()
@@ -277,7 +278,7 @@ class UIWall(Wall):
         uiTile.update()
 
     @afterQueuedAnimations
-    def _placeLooseTiles(self, deferredResult):
+    def _placeLooseTiles(self, deferredResult=None):
         """place the last 2 tiles on top of kong box"""
         assert len(self.kongBox) % 2 == 0
         placeCount = len(self.kongBox) // 2
