@@ -27,19 +27,19 @@ import os
 import datetime
 import time
 import subprocess
+import gc
 
 from locale import getpreferredencoding
 from sys import stdout
+
+from common import Debug, isPython3, unicode
+
 try:
     STDOUTENCODING = stdout.encoding
 except AttributeError:
     STDOUTENCODING = None
 if not STDOUTENCODING:
-    STDOUTENCODING = getpreferredencoding()
-
-# util must not depend on kde
-
-from common import Debug, isPython3
+    STDOUTENCODING = getpreferredencoding() # pylint: disable=redefined-variable-type
 
 
 def stack(msg, limit=6):
@@ -56,14 +56,23 @@ def stack(msg, limit=6):
     return result
 
 
-def callers(count=1, exclude=None):
+def callers(count=5, exclude=None):
     """returns the name of the calling method"""
     stck = traceback.extract_stack(limit=30)
-    excluding = list(exclude) or []
+    excluding = list(exclude) if exclude else []
     excluding.extend(['<genexpr>', '__call__', 'run', '<module>', 'runTests'])
-    names = list(x[2] for x in stck[:-2] if x[2] not in excluding)
-    result = '.'.join(names[-count - 2:])
-    return result
+    excluding.extend(['_startRunCallbacks', '_runCallbacks', 'remote_move', 'exec_move'])
+    excluding.extend(['proto_message', '_recvMessage', 'remoteMessageReceived'])
+    excluding.extend(['clientAction', 'myAction', 'expressionReceived'])
+    excluding.extend(['callbackIfDone', 'callback', '__gotAnswer'])
+    excluding.extend(['callExpressionReceived', 'proto_answer'])
+    excluding.extend(['_dataReceived', 'dataReceived', 'gotItem'])
+    excluding.extend(['callWithContext', '_doReadOrWrite', 'doRead'])
+    excluding.extend(['callers', 'debug'])
+    _ = list(([x[2] for x in stck if x[2] not in excluding]))
+    names = reversed(_[-count:])
+    result = '.'.join(names)
+    return '[{}]'.format(result)
 
 
 def elapsedSince(since):
@@ -96,8 +105,6 @@ def uniqueList(seq):
     seen_add = seen.add
     return [x for x in seq if x not in seen and not seen_add(x)]
 
-import gc
-
 
 def _getr(slist, olist, seen):
     """Recursively expand slist's objects into olist, using seen to track
@@ -147,7 +154,6 @@ def kprint(*args, **kwargs):
             arg = repr(arg)
         arg = arg.encode(STDOUTENCODING, 'ignore')
         newArgs.append(arg)
-    # we need * magic: pylint: disable=star-args
     try:
         print(*newArgs, sep=kwargs.get('sep', ' '),
               end=kwargs.get('end', '\n'), file=kwargs.get('file'))
@@ -204,7 +210,7 @@ def checkMemory():
     print('}}} done')
 
     # code like this may help to find specific things
-    if True:
+    if True: # pylint: disable=using-constant-test
         interesting = ('Client', 'Player', 'Game')
         for obj in gc.garbage:
             if hasattr(obj, 'cell_contents'):
