@@ -84,10 +84,12 @@ class Wall(object):
         """define what tile this is"""
         if element is None:
             return tile
-        elif isinstance(tile, Tile):
-            return Tile(element)
+        assert isinstance(element, Tile), element
+        if isinstance(tile, Tile):
+            return element
         else:
-            tile.tile = Tile(element)
+            # tile is UITile
+            tile.tile = element
             return tile
 
     def deal(self, tiles=None, deadEnd=False):
@@ -106,7 +108,7 @@ class Wall(object):
             dealTiles = self.living[:count]
             self.living = self.living[count:]
         return list(self.__nameTile(*x)
-                    for x in zip(dealTiles, tiles))  # pylint: disable=W0142
+                    for x in zip(dealTiles, tiles))
 
     def build(self, shuffleFirst=False):
         """virtual: build visible wall"""
